//+------------------------------------------------------------------+
//|                                           StochCandles_80_20.mq4 |
//|                                                  modified by cja |
//+------------------------------------------------------------------+
#property copyright "Copyright  2007, Christof Risch (iya)"
#property link      ""
#property indicator_chart_window
#property indicator_buffers 4
#property indicator_color1  clrGreen//wick
#property indicator_color2  clrRed//wick
#property indicator_color3  clrGreen//candle
#property indicator_color4  clrRed//candle
#property indicator_width1  1
#property indicator_width2  1
#property indicator_width3  3
#property indicator_width4  3


extern ENUM_TIMEFRAMES TimeFrame       = PERIOD_CURRENT;
extern int		        Stoch_K		   = 14;
extern int				  Stoch_D		   = 7;
extern int				  Stoch_Slowing   = 5;
extern ENUM_STO_PRICE  StoPrice        = 0;					
extern double		     Overbought	   = 80.0;
extern double  		  Oversold		   = 20.0;
extern string          note            = "turn on Alert = true; turn off = false";
extern bool            alertsOn        = true;
extern bool            alertsOnCurrent = false;
extern bool            alertsMessage   = true;
extern bool            alertsSound     = true;
extern bool            alertsEmail     = false;
extern bool            alertsNotify    = false;
extern string          soundFile       = "alert2.wav";

//---- input parameters
extern int		       BarWidth			= 1,
					       CandleWidth		= 3;

//---- buffers
double Bar1[],
		 Bar2[],
		 Candle1[],
		 Candle2[],
		 trend[];
string indicatorFileName;
bool   returnBars;
		 

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
//---- indicators
	IndicatorShortName("Colored Candles: Stoch("+Stoch_K+","+Stoch_D+","+Stoch_Slowing+")");
	IndicatorBuffers(5);
	SetIndexBuffer(0,Bar1);    SetIndexStyle(0,DRAW_HISTOGRAM,0,BarWidth);
	SetIndexBuffer(1,Bar2);		SetIndexStyle(1,DRAW_HISTOGRAM,0,BarWidth);		
	SetIndexBuffer(2,Candle1); SetIndexStyle(2,DRAW_HISTOGRAM,0,CandleWidth);
	SetIndexBuffer(3,Candle2); SetIndexStyle(3,DRAW_HISTOGRAM,0,CandleWidth);
	SetIndexBuffer(4,trend);
   	indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame==-99;
      TimeFrame         = MathMax(TimeFrame,_Period);
	return(0);
}

//+------------------------------------------------------------------+

void SetCandleColor(int col, int i)
{
	double high,low,bodyHigh,bodyLow;
	
	bodyHigh = MathMax(Open[i],Close[i]);
   bodyLow  = MathMin(Open[i],Close[i]);
   high		= High[i];
   low		= Low[i];

	Bar1[i]    = EMPTY_VALUE;
   Bar2[i]    = EMPTY_VALUE;
   Candle1[i] = EMPTY_VALUE;
   Candle2[i] = EMPTY_VALUE;
	
	switch(col)
	{
		case 1: Bar1[i] = high;	Bar2[i] = low; Candle1[i] = bodyHigh;	Candle2[i] = bodyLow; break;
      case 2: Bar2[i] = high;	Bar1[i] = low; Candle2[i] = bodyHigh;	Candle1[i] = bodyLow; break;
	
	}
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   int counted_bars=IndicatorCounted();
   if (counted_bars<0) return(-1);
   if (counted_bars>0) counted_bars--;
         int limit=MathMin(Bars-counted_bars,Bars-1);
         if (returnBars)  { Bar1[0] = limit+1; return(0); }
         if (TimeFrame!=_Period)
         {
            limit = MathMax(limit,MathMin(Bars,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
            for(int i = limit; i >= 0; i--)
            {
               int y = iBarShift(NULL,TimeFrame,Time[i]);
               trend[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,Stoch_K,Stoch_D,Stoch_Slowing,StoPrice,Overbought,Oversold,"",alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotify,soundFile,4,y);
               if (trend[i] ==  1) SetCandleColor(1,i); 
               if (trend[i] == -1) SetCandleColor(2,i); 
            }
            return(0);
         }
   
   //
   //
   //
   //
   //
   
	for(i = limit; i>=0; i--)//MathMax(Bars-1-IndicatorCounted(),1)
	{
		double sto	= iStochastic(NULL,0,Stoch_K,Stoch_D,Stoch_Slowing,MODE_SMA,StoPrice,MODE_MAIN,i); 
		       trend[i] = 0;
			 	 if(sto > Overbought)	{ trend[i] = 1; SetCandleColor(1,i); }
		else	 if(sto < Oversold)		{ trend[i] =-1; SetCandleColor(2,i); }
		
	}
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert("overbought");
         if (trend[whichBar] == -1) doAlert("oversold");       
      }         
   }
   return(0);
}

//+------------------------------------------------------------------+


void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," StochCandles ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," StochCandles "),message);
             if (alertsNotify)  SendNotification(message);
             if (alertsSound)   PlaySound(soundFile);
      }
}
